﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using EasyPortal.Web.Weibo.Sina.Properties;
using System.Web;
using System.Collections.Specialized;
using System.Net;
using System.IO;

namespace EasyPortal.Web.Weibo.Sina
{
    /// <summary>
    /// Sina weibo api
    /// </summary>
    public sealed class SinaApi : BasicSDK
    {

        /// <summary>
        /// Initializes a new instance of the <see cref="SinaApi"/> class.
        /// </summary>
        /// <param name="oauth">The OAuth object.</param>
        /// <param name="credentialPersistence">The ICredentialPersistence instance.</param>
        /// <param name="responseType">Type of the response.</param>
        public SinaApi(OAuth oauth, ICredentialPersistence credentialPersistence, ResponseType responseType) : base(oauth, credentialPersistence, responseType) { }


        /// <summary>
        /// Gets the encoding.
        /// </summary>
        public override Encoding Encoding
        {
            get { return Encoding.GetEncoding("iso-8859-1"); }
        }

        /// <summary>
        /// Gets a new instance of the <see cref="SinaApi"/> class use the ResponseType.Xml.
        /// </summary>
        public static SinaApi Xml
        {
            get
            {
                return new SinaApi(new OAuth(), new SessionCredentialPersistence(), ResponseType.Xml);
            }
        }

        /// <summary>
        /// Gets a new instance of the <see cref="SinaApi"/> class use the ResponseType.Json.
        /// </summary>
        public static SinaApi Json
        {
            get
            {
                return new SinaApi(new OAuth(), new SessionCredentialPersistence(), ResponseType.Json);
            }
        }

        /// <summary>
        /// Create the request url with template and response type.
        /// </summary>
        /// <param name="urlTemplate">The url template.</param>
        /// <param name="restParams">The RESTful params.</param>
        /// <returns>
        /// The request url
        /// </returns>
        private string CreateRequestUrl(string urlTemplate, params string[] restParams)
        {
            Checker.Parameter(!string.IsNullOrWhiteSpace(urlTemplate), "url template can not be empty or null");

            string query = string.Empty;

            //for chinese urlencode twice
            foreach (string s in restParams)
                query += "/:" + UrlEncode(UrlEncode(s));

            if (ResponseType == ResponseType.Xml)
                urlTemplate += query + ".xml";
            if (ResponseType == ResponseType.Json)
                urlTemplate += query + ".json";

            return AppendQuery(urlTemplate, "source", OAuth.ConsumerCredential.Key);
        }

        /// <summary>
        /// URLs the encode.
        /// </summary>
        /// <param name="input">The input.</param>
        /// <returns></returns>
        protected override string UrlEncode(string input)
        {
            return base.UrlEncode(input).Replace("!", "%21").Replace("*", "%2A").Replace("(", "%28").Replace(")", "%29").Replace("'", "%27");
        }

        /// <summary>
        /// Append parameter to the end of url query string
        /// </summary>
        /// <param name="url">The url.</param>
        /// <param name="pName">The parameter name.</param>
        /// <param name="pValue">The parameter value.</param>
        /// <returns>
        /// New url with query string
        /// </returns>
        protected override string AppendQuery(string url, string pName, string pValue)
        {
            //for chinese urlencode twice
            return base.AppendQuery(url, pName, UrlEncode(pValue));
        }

        /// <summary>
        /// Gets the authorization url.
        /// </summary>
        /// <param name="callbackUrl">The callback url.</param>
        /// <returns>The authorization url</returns>
        public string GetAuthorizationUrl(string callbackUrl)
        {
            NameValueCollection additionalParams;
            return GetAuthorizationUrl(Resources.url_oauth_request_token, Resources.url_oauth_authorize, callbackUrl, out additionalParams);
        }

        /// <summary>
        /// Saves the access token.
        /// </summary>
        /// <param name="verifier">The verifier string.</param>
        /// <returns>
        /// If access token successfully saved return true, otherwise return false.
        /// </returns>
        public bool SaveAccessToken(string verifier)
        {
            string uid;
            return SaveAccessToken(verifier, out uid);
        }

        /// <summary>
        /// Saves the access token.
        /// </summary>
        /// <param name="verifier">The verifier string.</param>
        /// <param name="userId">The user id.</param>
        /// <returns>
        /// If access token successfully saved return true, otherwise return false.
        /// </returns>
        public bool SaveAccessToken(string verifier, out string userId)
        {
            userId = string.Empty;
            NameValueCollection additionalParams;
            if (SaveAccessTokenCredential(verifier, Resources.url_oauth_access_token, out additionalParams))
            {
                userId = additionalParams["user_id"];
                return true;
            }
            else
                return false;
        }

        /// <summary>
        /// Gets the base62 format of mid.
        /// </summary>
        /// <param name="mid">The message mid.</param>
        /// <returns>The base62 format of mid</returns>
        public string GetBase62Mid(long mid)
        {
            string midStr = mid.ToString();

            midStr = midStr.PadLeft(midStr.Length + (7 - midStr.Length % 7), '0');

            Base62Encoder encoder = new Base62Encoder();
            string midBase62 = string.Empty;
            for (int i = 0; i * 7 < midStr.Length; i++)
            {
                midBase62 += encoder.ToBase62String(long.Parse(midStr.Substring(i * 7, 7)));
            }
            return midBase62;
        }

        /// <summary>
        /// Gets the long value from mid base62 format.
        /// </summary>
        /// <param name="base62Mid">The base62 mid string.</param>
        /// <returns>The mid long value.</returns>
        public long GetLongMid(string base62Mid)
        {
            Checker.Parameter(!string.IsNullOrWhiteSpace(base62Mid), "base62 mid string can not be empty or null");

            base62Mid = base62Mid.PadLeft(base62Mid.Length + (4 - base62Mid.Length % 4), '*');

            Base62Encoder encoder = new Base62Encoder();
            string longString = string.Empty;

            for (int i = 0; i * 4 < base62Mid.Length; i++)
            {
                longString += encoder.FromBase62String(base62Mid.Substring(i * 4, 4).TrimStart('*')).ToString();
            }

            return long.Parse(longString);
        }


        /// <summary>
        /// Emotions
        /// </summary>
        /// <param name="type">The type.</param>
        /// <param name="language">The language.</param>
        /// <returns>The ResponseObj instance(never null).</returns>
        public ResponseObj Emotions(string type, string language)
        {
            string requestUrl = CreateRequestUrl(Resources.url_emotions);
            if (!string.IsNullOrWhiteSpace(type))
                requestUrl = AppendQuery(requestUrl, "type", type);
            if (!string.IsNullOrWhiteSpace(language))
                requestUrl = AppendQuery(requestUrl, "language", language);

            return HttpGet(requestUrl, false);
        }

        /// <summary>
        /// Public_Timeline
        /// </summary>
        /// <param name="count">The count.</param>
        /// <param name="base_app">if set to <c>true</c> [base_app].</param>
        /// <returns>The ResponseObj instance(never null).</returns>
        public ResponseObj Public_Timeline(int? count, bool? base_app)
        {
            string requestUrl = CreateRequestUrl(Resources.url_public_timeline);
            if (count != null)
                requestUrl = AppendQuery(requestUrl, "count", count.Value.ToString());
            if (base_app != null)
                requestUrl = AppendQuery(requestUrl, "base_app", base_app.Value ? "1" : "0");

            return HttpGet(requestUrl, false);
        }

        /// <summary>
        /// Users_Show.
        /// </summary>
        /// <param name="id">The id.</param>
        /// <returns>The ResponseObj instance(never null).</returns>
        public ResponseObj Users_Show(string id)
        {
            Checker.Parameter(!string.IsNullOrWhiteSpace(id), "id can not be empty or null");

            string requestUrl = CreateRequestUrl(Resources.url_users_show, id);

            return HttpGet(requestUrl);
        }

        /// <summary>
        /// Statuses_Comments_Timeline
        /// </summary>
        /// <param name="since_id">The since_id.</param>
        /// <param name="max_id">The max_id.</param>
        /// <param name="count">The count.</param>
        /// <param name="page">The page.</param>
        /// <returns>The ResponseObj instance(never null).</returns>
        public ResponseObj Statuses_Comments_Timeline(long? since_id, long? max_id, int? count, int? page)
        {
            string requestUrl = CreateRequestUrl(Resources.url_statuses_comments_timeline);
            if (since_id != null)
                requestUrl = AppendQuery(requestUrl, "since_id", since_id.Value.ToString());
            if (max_id != null)
                requestUrl = AppendQuery(requestUrl, "max_id", max_id.Value.ToString());
            if (count != null)
                requestUrl = AppendQuery(requestUrl, "count", count.Value.ToString());
            if (page != null)
                requestUrl = AppendQuery(requestUrl, "page", page.Value.ToString());

            return HttpGet(requestUrl);
        }

        /// <summary>
        /// Statuses_Comments_By_Me.
        /// </summary>
        /// <param name="since_id">The since_id.</param>
        /// <param name="max_id">The max_id.</param>
        /// <param name="count">The count.</param>
        /// <param name="page">The page.</param>
        /// <returns>The ResponseObj instance(never null).</returns>
        public ResponseObj Statuses_Comments_By_Me(long? since_id, long? max_id, int? count, int? page)
        {
            string requestUrl = CreateRequestUrl(Resources.url_statuses_comments_by_me);
            if (since_id != null)
                requestUrl = AppendQuery(requestUrl, "since_id", since_id.Value.ToString());
            if (max_id != null)
                requestUrl = AppendQuery(requestUrl, "max_id", max_id.Value.ToString());
            if (count != null)
                requestUrl = AppendQuery(requestUrl, "count", count.Value.ToString());
            if (page != null)
                requestUrl = AppendQuery(requestUrl, "page", page.Value.ToString());

            return HttpGet(requestUrl);
        }

        /// <summary>
        /// Statuses_Comments_To_Me.
        /// </summary>
        /// <param name="since_id">The since_id.</param>
        /// <param name="max_id">The max_id.</param>
        /// <param name="count">The count.</param>
        /// <param name="page">The page.</param>
        /// <returns>The ResponseObj instance(never null).</returns>
        public ResponseObj Statuses_Comments_To_Me(long? since_id, long? max_id, int? count, int? page)
        {
            string requestUrl = CreateRequestUrl(Resources.url_statuses_comments_to_me);
            if (since_id != null)
                requestUrl = AppendQuery(requestUrl, "since_id", since_id.Value.ToString());
            if (max_id != null)
                requestUrl = AppendQuery(requestUrl, "max_id", max_id.Value.ToString());
            if (count != null)
                requestUrl = AppendQuery(requestUrl, "count", count.Value.ToString());
            if (page != null)
                requestUrl = AppendQuery(requestUrl, "page", page.Value.ToString());

            return HttpGet(requestUrl);
        }

        /// <summary>
        /// Statuses_Repost_Timeline.
        /// </summary>
        /// <param name="id">The weibo message id.</param>
        /// <param name="since_id">The since_id.</param>
        /// <param name="max_id">The max_id.</param>
        /// <param name="count">The count.</param>
        /// <param name="page">The page.</param>
        /// <returns>The ResponseObj instance(never null).</returns>
        public ResponseObj Statuses_Repost_Timeline(long id, long? since_id, long? max_id, int? count, int? page)
        {
            string requestUrl = CreateRequestUrl(Resources.url_statuses_repost_timeline);
            requestUrl = AppendQuery(requestUrl, "id", id.ToString());
            if (since_id != null)
                requestUrl = AppendQuery(requestUrl, "since_id", since_id.Value.ToString());
            if (max_id != null)
                requestUrl = AppendQuery(requestUrl, "max_id", max_id.Value.ToString());
            if (count != null)
                requestUrl = AppendQuery(requestUrl, "count", count.Value.ToString());
            if (page != null)
                requestUrl = AppendQuery(requestUrl, "page", page.Value.ToString());
            return HttpGet(requestUrl);
        }

        /// <summary>
        /// Statuses_Repost_By_Me.
        /// </summary>
        /// <param name="id">The user id.</param>
        /// <param name="since_id">The since_id.</param>
        /// <param name="max_id">The max_id.</param>
        /// <param name="count">The count.</param>
        /// <param name="page">The page.</param>
        /// <returns>The ResponseObj instance(never null).</returns>
        public ResponseObj Statuses_Repost_By_Me(long id, long? since_id, long? max_id, int? count, int? page)
        {
            string requestUrl = CreateRequestUrl(Resources.url_statuses_repost_by_me);
            requestUrl = AppendQuery(requestUrl, "id", id.ToString());
            if (since_id != null)
                requestUrl = AppendQuery(requestUrl, "since_id", since_id.Value.ToString());
            if (max_id != null)
                requestUrl = AppendQuery(requestUrl, "max_id", max_id.Value.ToString());
            if (count != null)
                requestUrl = AppendQuery(requestUrl, "count", count.Value.ToString());
            if (page != null)
                requestUrl = AppendQuery(requestUrl, "page", page.Value.ToString());
            return HttpGet(requestUrl);
        }

        /// <summary>
        /// Statuses_Mentions.
        /// </summary>
        /// <param name="since_id">The since_id.</param>
        /// <param name="max_id">The max_id.</param>
        /// <param name="count">The count.</param>
        /// <param name="page">The page.</param>
        /// <returns>The ResponseObj instance(never null).</returns>
        public ResponseObj Statuses_Mentions(long? since_id, long? max_id, int? count, int? page)
        {
            string requestUrl = CreateRequestUrl(Resources.url_statuses_mentions);
            if (since_id != null)
                requestUrl = AppendQuery(requestUrl, "since_id", since_id.Value.ToString());
            if (max_id != null)
                requestUrl = AppendQuery(requestUrl, "max_id", max_id.Value.ToString());
            if (count != null)
                requestUrl = AppendQuery(requestUrl, "count", count.Value.ToString());
            if (page != null)
                requestUrl = AppendQuery(requestUrl, "page", page.Value.ToString());

            return HttpGet(requestUrl);
        }

        /// <summary>
        /// Friends_Ids.
        /// </summary>
        /// <param name="id">The id.</param>
        /// <param name="count">The count.</param>
        /// <param name="cursor">The cursor.</param>
        /// <returns>The ResponseObj instance(never null).</returns>
        public ResponseObj Friends_Ids(string id, int? count, int? cursor)
        {
            Checker.Parameter(!string.IsNullOrWhiteSpace(id), "id can not be empty or null");

            string requestUrl = CreateRequestUrl(Resources.url_friends_ids, id);
            if (count != null)
                requestUrl = AppendQuery(requestUrl, "count", count.Value.ToString());
            if (cursor != null)
                requestUrl = AppendQuery(requestUrl, "cursor", cursor.Value.ToString());

            return HttpGet(requestUrl);
        }

        /// <summary>
        /// Followers_Ids.
        /// </summary>
        /// <param name="id">The id.</param>
        /// <param name="count">The count.</param>
        /// <param name="cursor">The cursor.</param>
        /// <returns>The ResponseObj instance(never null).</returns>
        public ResponseObj Followers_Ids(string id, int? count, int? cursor)
        {
            Checker.Parameter(!string.IsNullOrWhiteSpace(id), "id can not be empty or null");

            string requestUrl = CreateRequestUrl(Resources.url_followers_ids, id);
            if (count != null)
                requestUrl = AppendQuery(requestUrl, "count", count.Value.ToString());
            if (cursor != null)
                requestUrl = AppendQuery(requestUrl, "cursor", cursor.Value.ToString());

            return HttpGet(requestUrl);
        }

        /// <summary>
        /// Statuses_Friends.
        /// </summary>
        /// <param name="id">The id.</param>
        /// <param name="count">The count.</param>
        /// <param name="cursor">The cursor.</param>
        /// <returns>The ResponseObj instance(never null).</returns>
        public ResponseObj Statuses_Friends(string id, int? count, int? cursor)
        {
            Checker.Parameter(!string.IsNullOrWhiteSpace(id), "id can not be empty or null");

            string requestUrl = CreateRequestUrl(Resources.url_statuses_friends, id);
            if (count != null)
                requestUrl = AppendQuery(requestUrl, "count", count.Value.ToString());
            if (cursor != null)
                requestUrl = AppendQuery(requestUrl, "cursor", cursor.Value.ToString());

            return HttpGet(requestUrl);
        }

        /// <summary>
        /// Statuses_Followers.
        /// </summary>
        /// <param name="id">The id.</param>
        /// <param name="count">The count.</param>
        /// <param name="cursor">The cursor.</param>
        /// <returns>The ResponseObj instance(never null).</returns>
        public ResponseObj Statuses_Followers(string id, int? count, int? cursor)
        {
            Checker.Parameter(!string.IsNullOrWhiteSpace(id), "id can not be empty or null");

            string requestUrl = CreateRequestUrl(Resources.url_statuses_followers, id);
            if (count != null)
                requestUrl = AppendQuery(requestUrl, "count", count.Value.ToString());
            if (cursor != null)
                requestUrl = AppendQuery(requestUrl, "cursor", cursor.Value.ToString());

            return HttpGet(requestUrl);
        }

        /// <summary>
        /// Statuses_User_Timeline.
        /// </summary>
        /// <param name="id">The id.</param>
        /// <param name="since_id">The since_id.</param>
        /// <param name="max_id">The max_id.</param>
        /// <param name="count">The count.</param>
        /// <param name="page">The page.</param>
        /// <param name="base_app">The base_app.</param>
        /// <param name="feature">The feature.</param>
        /// <returns>The ResponseObj instance(never null).</returns>
        public ResponseObj Statuses_User_Timeline(string id, long? since_id, long? max_id, int? count, int? page, int? base_app, int? feature)
        {
            Checker.Parameter(!string.IsNullOrWhiteSpace(id), "id can not be empty or null");

            string requestUrl = CreateRequestUrl(Resources.url_statuses_user_timeline, id);
            if (since_id != null)
                requestUrl = AppendQuery(requestUrl, "since_id", since_id.Value.ToString());
            if (max_id != null)
                requestUrl = AppendQuery(requestUrl, "max_id", max_id.Value.ToString());
            if (count != null)
                requestUrl = AppendQuery(requestUrl, "count", count.Value.ToString());
            if (page != null)
                requestUrl = AppendQuery(requestUrl, "page", page.Value.ToString());
            if (base_app != null)
                requestUrl = AppendQuery(requestUrl, "base_app", base_app.Value.ToString());
            if (feature != null)
                requestUrl = AppendQuery(requestUrl, "feature", feature.Value.ToString());

            return HttpGet(requestUrl);
        }


        /// <summary>
        /// Statuses_Repost.
        /// </summary>
        /// <param name="id">The id.</param>
        /// <param name="status">The status.</param>
        /// <param name="is_comment">The is_comment.</param>
        /// <returns>The ResponseObj instance(never null).</returns>
        public ResponseObj Statuses_Repost(long id, string status, int? is_comment)
        {
            string requestUrl = CreateRequestUrl(Resources.url_statuses_repost);
            requestUrl = AppendQuery(requestUrl, "id", id.ToString());
            if (!string.IsNullOrWhiteSpace(status))
                requestUrl = AppendQuery(requestUrl, "status", status);
            if (is_comment != null)
                requestUrl = AppendQuery(requestUrl, "is_comment", is_comment.Value.ToString());

            return HttpPost(requestUrl);
        }

        /// <summary>
        /// Statuses_Update
        /// </summary>
        /// <param name="status">The status.</param>
        /// <param name="in_reply_to_status_id">The in_reply_to_status_id.</param>
        /// <param name="lat">The lat.</param>
        /// <param name="longArg">The long.</param>
        /// <param name="annotations">The annotations.</param>
        /// <returns>
        /// The ResponseObj instance(never null).
        /// </returns>
        public ResponseObj Statuses_Update(string status, int? in_reply_to_status_id, float? lat, float? longArg, string annotations)
        {
            Checker.Parameter(!string.IsNullOrWhiteSpace(status), "status can not be empty or null");

            string requestUrl = CreateRequestUrl(Resources.url_statuses_update);
            requestUrl = AppendQuery(requestUrl, "status", status);
            if (in_reply_to_status_id != null)
                requestUrl = AppendQuery(requestUrl, "in_reply_to_status_id", in_reply_to_status_id.Value.ToString());
            if (lat != null)
                requestUrl = AppendQuery(requestUrl, "lat", lat.Value.ToString());
            if (longArg != null)
                requestUrl = AppendQuery(requestUrl, "long", longArg.Value.ToString());
            if (!string.IsNullOrWhiteSpace(annotations))
                requestUrl = AppendQuery(requestUrl, "annotations", annotations);

            return HttpPost(requestUrl);
        }

        /// <summary>
        /// Statuses_Upload.
        /// </summary>
        /// <param name="status">The status.</param>
        /// <param name="picFileName">The pic file name.</param>
        /// <param name="picContent">The pic file content.</param>
        /// <param name="picContentType">The pic file content type.</param>
        /// <param name="lat">The lat.</param>
        /// <param name="longArg">The long arg.</param>
        /// <returns>The ResponseObj instance(never null)</returns>
        public ResponseObj Statuses_Upload(string status, string picFileName, string picContent, string picContentType, float? lat, float? longArg)
        {
            Checker.Parameter(!string.IsNullOrWhiteSpace(status), "status can not be empty or null");
            Checker.Parameter(!string.IsNullOrWhiteSpace(picFileName), "picFileName can not be empty or null");
            Checker.Parameter(!string.IsNullOrWhiteSpace(picContent), "picContent can not be empty or null");
            Checker.Parameter(!string.IsNullOrWhiteSpace(picContentType), "picContentType can not be empty or null");

            string requestUrl = CreateRequestUrl(Resources.url_statuses_upload);
            requestUrl = AppendQuery(requestUrl, "status", status);
            if (lat != null)
                requestUrl = AppendQuery(requestUrl, "lat", lat.Value.ToString());
            if (longArg != null)
                requestUrl = AppendQuery(requestUrl, "long", longArg.Value.ToString());

            FilePostData fileData = new FilePostData(picFileName, picContent, "pic", picContentType);

            return HttpPost(requestUrl, fileData);
        }

        /// <summary>
        /// Statuses_Upload.
        /// </summary>
        /// <param name="status">The status.</param>
        /// <param name="postFile">The post file.</param>
        /// <param name="lat">The lat.</param>
        /// <param name="longArg">The long.</param>
        /// <returns>
        /// The ResponseObj instance(never null)
        /// </returns>
        public ResponseObj Statuses_Upload(string status, HttpPostedFileBase postFile, float? lat, float? longArg)
        {
            Checker.Parameter(!string.IsNullOrWhiteSpace(status), "status can not be empty or null");
            Checker.Parameter(postFile != null, "postFile can not be null");

            string requestUrl = CreateRequestUrl(Resources.url_statuses_upload);
            requestUrl = AppendQuery(requestUrl, "status", status);
            if (lat != null)
                requestUrl = AppendQuery(requestUrl, "lat", lat.Value.ToString());
            if (longArg != null)
                requestUrl = AppendQuery(requestUrl, "long", longArg.Value.ToString());

            FilePostData fileData = new FilePostData(postFile, Encoding, "pic");

            return HttpPost(requestUrl, fileData);
        }

        /// <summary>
        /// Statuses_Upload.
        /// </summary>
        /// <param name="status">The status.</param>
        /// <param name="postFile">The post file.</param>
        /// <param name="lat">The lat.</param>
        /// <param name="longArg">The long.</param>
        /// <returns>
        /// The ResponseObj instance(never null)
        /// </returns>
        public ResponseObj Statuses_Upload(string status, HttpPostedFile postFile, float? lat, float? longArg)
        {
            Checker.Parameter(!string.IsNullOrWhiteSpace(status), "status can not be empty or null");
            Checker.Parameter(postFile != null, "postFile can not be null");

            string requestUrl = CreateRequestUrl(Resources.url_statuses_upload);
            requestUrl = AppendQuery(requestUrl, "status", status);
            if (lat != null)
                requestUrl = AppendQuery(requestUrl, "lat", lat.Value.ToString());
            if (longArg != null)
                requestUrl = AppendQuery(requestUrl, "long", longArg.Value.ToString());

            FilePostData fileData = new FilePostData(postFile, Encoding, "pic");

            return HttpPost(requestUrl, fileData);
        }

        /// <summary>
        /// Statuses_Upload.
        /// </summary>
        /// <param name="status">The status.</param>
        /// <param name="picFileName">The pic file name.</param>
        /// <param name="picStream">The pic file stream.</param>
        /// <param name="picContentType">The pic file content type.</param>
        /// <param name="lat">The lat.</param>
        /// <param name="longArg">The long.</param>
        /// <returns>
        /// The ResponseObj instance(never null)
        /// </returns>
        public ResponseObj Statuses_Upload(string status, string picFileName, Stream picStream, string picContentType, float? lat, float? longArg)
        {
            Checker.Parameter(!string.IsNullOrWhiteSpace(status), "status can not be empty or null");
            Checker.Parameter(!string.IsNullOrWhiteSpace(picFileName), "picFileName can not be empty or null");
            Checker.Parameter(picStream != null, "picStream can not be null");
            Checker.Parameter(!string.IsNullOrWhiteSpace(picContentType), "picContentType can not be empty or null");

            string requestUrl = CreateRequestUrl(Resources.url_statuses_upload);
            requestUrl = AppendQuery(requestUrl, "status", status);
            if (lat != null)
                requestUrl = AppendQuery(requestUrl, "lat", lat.Value.ToString());
            if (longArg != null)
                requestUrl = AppendQuery(requestUrl, "long", longArg.Value.ToString());

            FilePostData fileData = new FilePostData(picFileName, picStream, Encoding, "pic", picContentType);

            return HttpPost(requestUrl, fileData);
        }

        /// <summary>
        /// Friendships_Create
        /// </summary>
        /// <param name="id">The id.</param>
        /// <returns>The ResponseObj instance(never null)</returns>
        public ResponseObj Friendships_Create(string id)
        {
            Checker.Parameter(!string.IsNullOrWhiteSpace(id), "id can not be empty or null");
            string requestUrl = CreateRequestUrl(Resources.url_friendships_create, id);

            return HttpPost(requestUrl);
        }

        /// <summary>
        /// Friendships_Exists.
        /// </summary>
        /// <param name="user_a">The user_a.</param>
        /// <param name="user_b">The user_b.</param>
        /// <returns>The ResponseObj instance(never null)</returns>
        public ResponseObj Friendships_Exists(long user_a, long user_b)
        {
            string requestUrl = CreateRequestUrl(Resources.url_friendships_exists);
            requestUrl = AppendQuery(requestUrl, "user_a", user_a.ToString());
            requestUrl = AppendQuery(requestUrl, "user_b", user_a.ToString());

            return HttpGet(requestUrl);
        }

        /// <summary>
        /// Friendships_Show.
        /// </summary>
        /// <param name="target_id">The target_id.</param>
        /// <param name="source_id">The source_id.</param>
        /// <returns>The ResponseObj instance(never null)</returns>
        public ResponseObj Friendships_Show(long target_id, long? source_id)
        {
            string requestUrl = CreateRequestUrl(Resources.url_friendships_show);
            requestUrl = AppendQuery(requestUrl, "target_id", target_id.ToString());
            if (source_id != null)
                requestUrl = AppendQuery(requestUrl, "source_id", source_id.Value.ToString());

            return HttpGet(requestUrl);
        }

        /// <summary>
        /// Get message url.
        /// </summary>
        /// <param name="userId">The user id.</param>
        /// <param name="msgId">The msg id.</param>
        /// <returns>The message url.</returns>
        public string GetMessageUrl(long userId, long msgId)
        {
            return Resources.url_user_statuses_id.Replace(":userid", userId.ToString()).Replace(":id", msgId.ToString());
        }

        /// <summary>
        /// Get message url from the mid.
        /// </summary>
        /// <param name="userId">The user id.</param>
        /// <param name="mid">The mid.</param>
        /// <returns>The message url.</returns>
        public string GetMessageUrlFromMid(long userId, long mid)
        {
            return Resources.url_user_statuses_id_by_mid.Replace(":userid", userId.ToString()).Replace(":mid", GetBase62Mid(mid));
        }


        /// <summary>
        /// Queryid.
        /// </summary>
        /// <param name="mid">The mid.</param>
        /// <param name="type">The type.</param>
        /// <param name="inbox">The inbox.</param>
        /// <param name="isBase62">The is base62.</param>
        /// <param name="is_batch">The is_batch.</param>
        /// <returns>The ResponseObj instance(never null)</returns>
        public ResponseObj Queryid(string mid, int type, int? inbox, bool? isBase62, bool? is_batch)
        {
            Checker.Parameter(!string.IsNullOrWhiteSpace(mid), "mid can not be empty or null");

            string requestUrl = CreateRequestUrl(Resources.url_queryid);
            requestUrl = AppendQuery(requestUrl, "mid", mid.Trim());
            requestUrl = AppendQuery(requestUrl, "type", type.ToString());
            if (inbox != null)
                requestUrl = AppendQuery(requestUrl, "inbox", inbox.Value.ToString());
            if (isBase62 != null)
                requestUrl = AppendQuery(requestUrl, "isBase62", isBase62.Value?"1":"0");
            if (is_batch != null)
                requestUrl = AppendQuery(requestUrl, "is_batch", is_batch.Value ? "1" : "0");

            return HttpGet(requestUrl, false);
        }
    }
}
