﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Web;
using System.Collections.Specialized;
using System.Net;
using System.Security.Cryptography;

namespace EasyPortal.Web.Weibo
{
    /// <summary>
    /// Weibo sdk basic functions
    /// </summary>
    public abstract class BasicSDK
    {
        OAuth _oauth;
        ICredentialPersistence _credentialPersistence;
        ResponseType _responseType;

        /// <summary>
        /// Initializes a new instance of the <see cref="BasicSDK"/> class.
        /// </summary>
        /// <param name="oauth">The OAuth object.</param>
        /// <param name="credentialPersistence">The ICredentialPersistence instance.</param>
        /// <param name="responseType">Type of the response.</param>
        public BasicSDK(OAuth oauth, ICredentialPersistence credentialPersistence, ResponseType responseType)
        {
            Checker.Parameter(oauth != null, "OAuth object can not be null");
            Checker.Parameter(credentialPersistence != null, "ICredentialPersistence instance can not be null");

            _oauth = oauth;
            _credentialPersistence = credentialPersistence;
            _responseType = responseType;
        }

        /// <summary>
        /// Gets the OAuth object.
        /// </summary>
        protected OAuth OAuth
        {
            get
            {
                return _oauth;
            }
        }

        /// <summary>
        /// Gets the credential persistence instance.
        /// </summary>
        public ICredentialPersistence CredentialPersistence
        {
            get
            {
                return _credentialPersistence;
            }
        }

        /// <summary>
        /// Gets the type of the response (xml or json).
        /// </summary>
        /// <value>
        /// The type of the response.
        /// </value>
        protected ResponseType ResponseType
        {
            get
            {
                return _responseType;
            }
        }

        /// <summary>
        /// Gets the encoding.
        /// </summary>
        public abstract Encoding Encoding { get; }

        /// <summary>
        /// Append parameter to the end of url query string
        /// </summary>
        /// <param name="url">The url.</param>
        /// <param name="pName">The parameter name.</param>
        /// <param name="pValue">The parameter value.</param>
        /// <returns>
        /// New url with query string
        /// </returns>
        protected virtual string AppendQuery(string url, string pName, string pValue)
        {
            Checker.Parameter(!string.IsNullOrWhiteSpace(url), "url can not be empty or null");
            Checker.Parameter(!string.IsNullOrWhiteSpace(pName), "parameter name can not be empty or null");

            string query = string.Format("{0}={1}", pName, UrlEncode(pValue));
            if (url.Contains('?'))
                return string.Join("&", url, query);
            return string.Join("?", url, query);
        }

        /// <summary>
        /// Gets the authorization url.
        /// </summary>
        /// <param name="requestApiUrl">The request api url</param>
        /// <param name="authUrl">The authorization api url</param>
        /// <param name="callbackUrl">The callback url.</param>
        /// <param name="additionalParams">The additional parameters callback from the api service.</param>
        /// <returns>
        /// The authorization url
        /// </returns>
        protected string GetAuthorizationUrl(string requestApiUrl, string authUrl, string callbackUrl, out NameValueCollection additionalParams)
        {
            Credential tempCredential = OAuth.GetRequestCredential(requestApiUrl, out additionalParams);
            CredentialPersistence.SetRequest(tempCredential);
            return OAuth.GetAuthorizationUrl(authUrl, tempCredential.Key, callbackUrl);
        }

        /// <summary>
        /// Saves the access token credential.
        /// </summary>
        /// <param name="verifier">The verifier string.</param>
        /// <param name="tokenApiUrl">The token api url.</param>
        /// <param name="additionalParams">The additional parameters callback from the api service.</param>
        /// <returns>
        /// If token credential successfully saved return true, otherwise return false.
        /// </returns>
        protected bool SaveAccessTokenCredential(string verifier, string tokenApiUrl, out NameValueCollection additionalParams)
        {
            additionalParams = new NameValueCollection();

            if (!string.IsNullOrEmpty(verifier))
            {
                Credential tokenCredential = OAuth.GetAccessCredential(tokenApiUrl, verifier, CredentialPersistence.GetRequest(), out additionalParams);
                CredentialPersistence.SetAccess(tokenCredential);
                CredentialPersistence.ClearRequest();
                return true;
            }
            else
                return false;
        }

        /// <summary>
        /// Normalizes the request parameters.
        /// </summary>
        /// <param name="requestUrl">The request URL.</param>
        /// <param name="httpMethod">The HTTP method.</param>
        /// <returns></returns>
        protected virtual string NormalizeRequestParameters(string requestUrl, HttpMethod httpMethod)
        {
            Checker.Parameter(!string.IsNullOrWhiteSpace(requestUrl), "requestUrl can not be empty or null");
            Checker.Parameter(httpMethod != null, "httpMethod can not be null");

            Uri reqUri = new Uri(requestUrl);
            Credential accessCredential = CredentialPersistence.GetAccess();
            string normalizedUrl = string.Empty;
            string normalizedRequestParameters = string.Empty;

            string sign = OAuth.GenerateSignature(reqUri, accessCredential.Key, accessCredential.Secret, httpMethod, out normalizedUrl, out normalizedRequestParameters);

            return normalizedRequestParameters += string.Format("&{0}={1}", OAuth.OAuthSignatureKey, UrlEncode(sign));
        }

        /// <summary>
        /// Creates the authorization request header.
        /// </summary>
        /// <param name="requestUrl">The request URL.</param>
        /// <param name="httpMethod">The http method.</param>
        /// <returns></returns>
        protected virtual NameValueCollection CreateAuthorizationRequestHeader(string requestUrl, HttpMethod httpMethod)
        {
            Checker.Parameter(!string.IsNullOrWhiteSpace(requestUrl), "requestUrl can not be empty or null");
            Checker.Parameter(httpMethod != null, "httpMethod can not be null");

            Uri reqUri = new Uri(requestUrl);

            string normalizedRequestParameters = NormalizeRequestParameters(requestUrl, httpMethod);

            foreach (string p in reqUri.Query.Trim('?').Split('&'))
                normalizedRequestParameters = normalizedRequestParameters.Replace("&" + p, string.Empty).Replace(p, string.Empty);

            normalizedRequestParameters = normalizedRequestParameters.Replace("=", "=\"").Replace("&", "\"&") + "\"";

            NameValueCollection collection = new NameValueCollection();

            collection.Add("Authorization", "OAuth " + string.Join(",", normalizedRequestParameters.Split(new char[] { '&' }, StringSplitOptions.RemoveEmptyEntries)));

            return collection;
        }

        /// <summary>
        /// Get the response use HTTP GET, and must authorization.
        /// </summary>
        /// <param name="requestUrl">The request url.</param>
        /// <returns>
        /// The ResponseObj instance(never null).
        /// </returns>
        protected ResponseObj HttpGet(string requestUrl)
        {
            return HttpGet(requestUrl, true);
        }


        /// <summary>
        /// Get the response use HTTP GET
        /// </summary>
        /// <param name="requestUrl">The request url.</param>
        /// <param name="needAuth">if set to <c>true</c> [need authorization].</param>
        /// <returns>
        /// The ResponseObj instance(never null).
        /// </returns>
        protected virtual ResponseObj HttpGet(string requestUrl, bool needAuth)
        {
            Checker.Parameter(!string.IsNullOrWhiteSpace(requestUrl), "requestUrl can not be empty or null");

            HttpWebRequest request = (HttpWebRequest)HttpWebRequest.Create(requestUrl);

            if (needAuth)
                request.Headers.Add(CreateAuthorizationRequestHeader(requestUrl, HttpMethod.Get));

            return WebHost.HttpGet(request);
        }


        /// <summary>
        /// Get the response use HTTP POST.
        /// </summary>
        /// <param name="requestUrl">The request url.</param>
        /// <returns>
        /// The ResponseObj instance(never null).
        /// </returns>
        protected virtual ResponseObj HttpPost(string requestUrl)
        {
            Checker.Parameter(!string.IsNullOrWhiteSpace(requestUrl), "requestUrl can not be empty or null");

            HttpWebRequest request = (HttpWebRequest)HttpWebRequest.Create(requestUrl.Substring(0, requestUrl.IndexOf("?")));
            request.Headers.Add(CreateAuthorizationRequestHeader(requestUrl, HttpMethod.Post));

            return WebHost.HttpPost(request, requestUrl.Substring(requestUrl.IndexOf("?") + 1));

        }

        /// <summary>
        /// Get the response use HTTP POST.
        /// </summary>
        /// <param name="requestUrl">The request URL.</param>
        /// <param name="fileData">The post file data.</param>
        /// <returns>The ResponseObj instance(never null).</returns>
        protected virtual ResponseObj HttpPost(string requestUrl, FilePostData fileData)
        {
            Checker.Parameter(!string.IsNullOrWhiteSpace(requestUrl), "requestUrl can not be empty or null");
            Checker.Parameter(fileData != null, "fileData can not be null");

            HttpWebRequest request = (HttpWebRequest)HttpWebRequest.Create(requestUrl.Substring(0, requestUrl.IndexOf("?")));

            request.Headers.Add(CreateAuthorizationRequestHeader(requestUrl, HttpMethod.Post));

            NameValueCollection queryCollection = HttpKits.ResolveParameters(requestUrl.Substring(requestUrl.IndexOf("?") + 1));

            List<IPostData> dataList = new List<IPostData>();

            foreach (string key in queryCollection.Keys)
            {
                dataList.Add(new BasicPostData(key, queryCollection[key]));
            }

            dataList.Add(fileData);

            return WebHost.HttpPost(request, dataList.ToArray(), Encoding);
        }


        /// <summary>
        /// URLs the encode.
        /// </summary>
        /// <param name="input">The input.</param>
        /// <returns></returns>
        protected virtual string UrlEncode(string input)
        {
            //different encode from OAuth.UrlEncode
            return Uri.EscapeDataString(input);
        }
    }
}
