using System;
using System.Security.Cryptography;
using System.Collections.Generic;
using System.Text;
using System.Web;
using System.Configuration;
using System.Collections.Specialized;

namespace EasyPortal.Web.Weibo
{
    /// <summary>
    /// The class specify a set of methods about OAuth.
    /// </summary>
    public sealed class OAuth
    {
        private const string OAuthVersion = "1.0";
        private const string OAuthParameterPrefix = "oauth_";

        //
        // List of know and used oauth parameters' names
        //        
        public const string OAuthConsumerKeyKey = "oauth_consumer_key";
        public const string OAuthCallbackKey = "oauth_callback";
        public const string OAuthVersionKey = "oauth_version";
        public const string OAuthSignatureMethodKey = "oauth_signature_method";
        public const string OAuthSignatureKey = "oauth_signature";
        public const string OAuthTimestampKey = "oauth_timestamp";
        public const string OAuthNonceKey = "oauth_nonce";
        public const string OAuthTokenKey = "oauth_token";
        public const string OAuthTokenSecretKey = "oauth_token_secret";
        public const string OAuthVerifier = "oauth_verifier";

        private const string HMACSHA1SignatureType = "HMAC-SHA1";
        private const string PlainTextSignatureType = "PLAINTEXT";
        private const string RSASHA1SignatureType = "RSA-SHA1";

        private Random random = new Random();

        private string unreservedChars = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789-_.~";

        Credential _consumerCredential;

        /// <summary>
        /// Initializes a new instance of the <see cref="OAuth"/> class.
        /// </summary>
        /// <param name="consumerCredential">The consumer credential.</param>
        public OAuth(Credential consumerCredential)
        {
            _consumerCredential = consumerCredential;
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="OAuth"/> class.
        /// </summary>
        /// <param name="consumerKey">The consumer key.</param>
        /// <param name="consumerSecret">The consumer secret.</param>
        public OAuth(string consumerKey, string consumerSecret)
        {
            Checker.Parameter(!string.IsNullOrWhiteSpace(consumerKey), "consumerKey can not be empty or null");
            Checker.Parameter(!string.IsNullOrWhiteSpace(consumerSecret), "consumerSecret can not be empty or null");
            _consumerCredential = new Credential { Key = consumerKey.Trim(), Secret = consumerSecret.Trim() };
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="OAuth"/> class.
        /// </summary>
        public OAuth()
            : this(ConfigurationManager.AppSettings["consumerKey"], ConfigurationManager.AppSettings["consumerSecret"])
        { }

        /// <summary>
        /// Gets the consumer credential.
        /// </summary>
        public Credential ConsumerCredential
        {
            get
            {
                return _consumerCredential;
            }
        }

        /// <summary>
        /// Helper function to compute a hash value
        /// </summary>
        /// <param name="data">The data to hash</param>
        /// <param name="hashAlgorithm">The hashing algoirhtm used. If that algorithm needs some initialization, like HMAC and its derivatives, they should be initialized prior to passing it to this function</param>
        /// <returns>a Base64 string of the hash value</returns>
        private string ComputeHash(string data, HashAlgorithm hashAlgorithm)
        {
            if (hashAlgorithm == null)
            {
                throw new ArgumentNullException("hashAlgorithm");
            }

            if (string.IsNullOrEmpty(data))
            {
                throw new ArgumentNullException("data");
            }

            byte[] dataBuffer = System.Text.Encoding.ASCII.GetBytes(data);
            byte[] hashBytes = hashAlgorithm.ComputeHash(dataBuffer);

            return Convert.ToBase64String(hashBytes);
        }

        /// <summary>
        /// Internal function to cut out all non oauth query string parameters (all parameters not begining with "oauth_")
        /// </summary>
        /// <param name="parameters">The query string part of the Url</param>
        /// <returns>A list of QueryParameter each containing the parameter name and value</returns>
        private List<QueryParameter> GetQueryParameters(string parameters)
        {
            if (parameters.StartsWith("?"))
            {
                parameters = parameters.Remove(0, 1);
            }

            List<QueryParameter> result = new List<QueryParameter>();

            if (!string.IsNullOrEmpty(parameters))
            {
                string[] p = parameters.Split('&');
                foreach (string s in p)
                {
                    if (!string.IsNullOrEmpty(s) && !s.StartsWith(OAuthParameterPrefix))
                    {
                        if (s.IndexOf('=') > -1)
                        {
                            string[] temp = s.Split('=');
                            result.Add(new QueryParameter(temp[0], temp[1]));
                        }
                        else
                        {
                            result.Add(new QueryParameter(s, string.Empty));
                        }
                    }
                }
            }

            return result;
        }

        /// <summary>
        /// This is a different Url Encode implementation since the default .NET one outputs the percent encoding in lower case.
        /// While this is not a problem with the percent encoding spec, it is used in upper case throughout OAuth
        /// </summary>
        /// <param name="value">The value to Url encode</param>
        /// <returns>Returns a Url encoded string</returns>
        public string UrlEncode(string value)
        {
            StringBuilder result = new StringBuilder();

            foreach (char symbol in value)
            {
                if (unreservedChars.IndexOf(symbol) != -1)
                {
                    result.Append(symbol);
                }
                else
                {
                    result.Append('%' + String.Format("{0:X2}", (int)symbol));
                }
            }

            return result.ToString();
        }

        /// <summary>
        /// Normalizes the request parameters according to the spec
        /// </summary>
        /// <param name="parameters">The list of parameters already sorted</param>
        /// <returns>a string representing the normalized parameters</returns>
        private string NormalizeRequestParameters(IList<QueryParameter> parameters)
        {
            StringBuilder sb = new StringBuilder();
            QueryParameter p = null;
            for (int i = 0; i < parameters.Count; i++)
            {
                p = parameters[i];
                sb.AppendFormat("{0}={1}", p.Name, p.Value);

                if (i < parameters.Count - 1)
                {
                    sb.Append("&");
                }
            }

            return sb.ToString();
        }

        /// <summary>
        /// Generate the signature base that is used to produce the signature
        /// </summary>
        /// <param name="url">The full url that needs to be signed including its non OAuth url parameters</param>
        /// <param name="token">The token, if available. If not available pass null or an empty string</param>
        /// <param name="tokenSecret">The token secret, if available. If not available pass null or an empty string</param>
        /// <param name="verifier">The verifier, if available. If not available pass null or an empty string</param>
        /// <param name="httpMethod">The http method used. Must be a valid HTTP method verb (POST,GET,PUT, etc)</param>
        /// <param name="signatureType">The signature type. To use the default values use <see cref="SignatureTypes">SignatureTypes</see>.</param>
        /// <param name="normalizedUrl">The normalized URL.</param>
        /// <param name="normalizedRequestParameters">The normalized request parameters.</param>
        /// <returns>
        /// The signature base string
        /// </returns>
        private string GenerateSignatureBaseString(Uri url, string token, string tokenSecret, string verifier, HttpMethod httpMethod, string signatureType, out string normalizedUrl, out string normalizedRequestParameters)
        {
            if (token == null)
            {
                token = string.Empty;
            }

            if (tokenSecret == null)
            {
                tokenSecret = string.Empty;
            }

            if (verifier == null)
                verifier = string.Empty;

            if ( null== httpMethod)
            {
                throw new ArgumentNullException("httpMethod");
            }

            if (string.IsNullOrEmpty(signatureType))
            {
                throw new ArgumentNullException("signatureType");
            }

            normalizedUrl = null;
            normalizedRequestParameters = null;

            List<QueryParameter> parameters = GetQueryParameters(url.Query);
            parameters.Add(new QueryParameter(OAuthVersionKey, OAuthVersion));
            parameters.Add(new QueryParameter(OAuthNonceKey, GenerateNonce()));
            parameters.Add(new QueryParameter(OAuthTimestampKey, GenerateTimeStamp()));
            parameters.Add(new QueryParameter(OAuthSignatureMethodKey, signatureType));
            parameters.Add(new QueryParameter(OAuthConsumerKeyKey, ConsumerCredential.Key));

            if (!string.IsNullOrEmpty(verifier) && httpMethod == HttpMethod.Get)
            {
                parameters.Add(new QueryParameter(OAuthVerifier, verifier));
            }

            if (!string.IsNullOrEmpty(token))
            {
                parameters.Add(new QueryParameter(OAuthTokenKey, token));
            }

            parameters.Sort(new QueryParameterComparer());

            normalizedUrl = string.Format("{0}://{1}", url.Scheme, url.Host);
            if (!((url.Scheme == "http" && url.Port == 80) || (url.Scheme == "https" && url.Port == 443)))
            {
                normalizedUrl += ":" + url.Port;
            }
            normalizedUrl += url.AbsolutePath;
            normalizedRequestParameters = NormalizeRequestParameters(parameters);

            StringBuilder signatureBase = new StringBuilder();
            signatureBase.AppendFormat("{0}&", httpMethod.Name);
            signatureBase.AppendFormat("{0}&", UrlEncode(normalizedUrl));
            signatureBase.AppendFormat("{0}", UrlEncode(normalizedRequestParameters));

            return signatureBase.ToString();
        }

        /// <summary>
        /// Generates a signature using the HMAC-SHA1 algorithm
        /// </summary>		
        /// <param name="url">The full url that needs to be signed including its non OAuth url parameters</param>
        /// <param name="httpMethod">The http method used. Must be a valid HTTP method verb (POST,GET,PUT, etc)</param>
        /// <returns>A base64 string of the hash value</returns>
        public string GenerateSignature(Uri url, HttpMethod httpMethod, out string normalizedUrl, out string normalizedRequestParameters)
        {
            return GenerateSignature(url, string.Empty, string.Empty, httpMethod, out normalizedUrl, out normalizedRequestParameters);
        }

        /// <summary>
        /// Generates a signature using the HMAC-SHA1 algorithm
        /// </summary>		
        /// <param name="url">The full url that needs to be signed including its non OAuth url parameters</param>
        /// <param name="token">The token, if available. If not available pass null or an empty string</param>
        /// <param name="tokenSecret">The token secret, if available. If not available pass null or an empty string</param>
        /// <param name="httpMethod">The http method used. Must be a valid HTTP method verb (POST,GET,PUT, etc)</param>
        /// <returns>A base64 string of the hash value</returns>
        public string GenerateSignature(Uri url, string token, string tokenSecret, HttpMethod httpMethod, out string normalizedUrl, out string normalizedRequestParameters)
        {
            return GenerateSignature(url, token, tokenSecret, string.Empty, httpMethod, SignatureTypes.HMACSHA1, out normalizedUrl, out normalizedRequestParameters);
        }

        /// <summary>
        /// Generates a signature using the specified signatureType
        /// </summary>
        /// <param name="url">The full url that needs to be signed including its non OAuth url parameters</param>
        /// <param name="token">The token, if available. If not available pass null or an empty string</param>
        /// <param name="tokenSecret">The token secret, if available. If not available pass null or an empty string</param>
        /// <param name="verifier">The verifier, if available. If not available pass null or an empty string</param>
        /// <param name="httpMethod">The http method used. Must be a valid HTTP method verb (POST,GET,PUT, etc)</param>
        /// <param name="signatureType">The type of signature to use</param>
        /// <param name="normalizedUrl">The normalized URL.</param>
        /// <param name="normalizedRequestParameters">The normalized request parameters.</param>
        /// <returns>
        /// A base64 string of the hash value
        /// </returns>
        public string GenerateSignature(Uri url, string token, string tokenSecret, string verifier, HttpMethod httpMethod, SignatureTypes signatureType, out string normalizedUrl, out string normalizedRequestParameters)
        {
            normalizedUrl = null;
            normalizedRequestParameters = null;

            switch (signatureType)
            {
                case SignatureTypes.PLAINTEXT:
                    return HttpUtility.UrlEncode(string.Format("{0}&{1}", ConsumerCredential.Secret, tokenSecret));
                case SignatureTypes.HMACSHA1:
                    string baseString = GenerateSignatureBaseString(url, token, tokenSecret, verifier, httpMethod, HMACSHA1SignatureType, out normalizedUrl, out normalizedRequestParameters);

                    HMACSHA1 hmacsha1 = new HMACSHA1();
                    hmacsha1.Key = Encoding.ASCII.GetBytes(string.Format("{0}&{1}", UrlEncode(ConsumerCredential.Secret), string.IsNullOrEmpty(tokenSecret) ? "" : UrlEncode(tokenSecret)));

                    return ComputeHash(baseString, hmacsha1);
                case SignatureTypes.RSASHA1:
                    throw new NotImplementedException();
                default:
                    throw new ArgumentException("Unknown signature type", "signatureType");
            }
        }

        /// <summary>
        /// Generate the timestamp for the signature        
        /// </summary>
        /// <returns></returns>
        private string GenerateTimeStamp()
        {
            // Default implementation of UNIX time of the current UTC time
            TimeSpan ts = DateTime.UtcNow - new DateTime(1970, 1, 1, 0, 0, 0, 0);
            return Convert.ToInt64(ts.TotalSeconds).ToString();
        }

        /// <summary>
        /// Generate a nonce
        /// </summary>
        /// <returns></returns>
        private string GenerateNonce()
        {
            // Just a simple implementation of a random number between 123400 and 9999999
            return random.Next(123400, 9999999).ToString();
        }

        /// <summary>
        /// Gets the request credential.
        /// </summary>
        /// <param name="url">The api url.</param>
        /// <param name="additionalParams">The additional parameters callback from the api service.</param>
        /// <returns>The request credential</returns>
        public Credential GetRequestCredential(string url, out NameValueCollection additionalParams)
        {
            Checker.Parameter(!string.IsNullOrWhiteSpace(url), "url can not be empty or null");

            string normalizedUrl;
            string normalizedRequestParameters;
            string sign = GenerateSignature(new Uri(url), HttpMethod.Get, out normalizedUrl, out normalizedRequestParameters);

            string requestUrl = string.Format("{0}?{1}&{2}={3}", normalizedUrl, normalizedRequestParameters, OAuthSignatureKey, HttpUtility.UrlEncode(sign));

            ResponseObj respObj = WebHost.HttpGet(requestUrl);

            Checker.Requires(respObj.Code == System.Net.HttpStatusCode.OK, "request error, code: {0} text: {1}", respObj.Code, respObj.Text);

            NameValueCollection paramCollection = HttpKits.ResolveParameters(respObj.Text);

            Checker.Requires(paramCollection.Count >= 2, "response not match the protocol, text: {0}", respObj.Text);

            Credential c = new Credential { Key = paramCollection[OAuthTokenKey], Secret = paramCollection[OAuthTokenSecretKey] };

            paramCollection.Remove(OAuthTokenKey);
            paramCollection.Remove(OAuthTokenSecretKey);

            additionalParams = paramCollection;

            return c;
        }

        /// <summary>
        /// Gets the authorization url.
        /// </summary>
        /// <param name="authUrl">The authorization api url.</param>
        /// <param name="requestKey">The request credential key.</param>
        /// <param name="callbackUrl">The callback url.</param>
        /// <returns></returns>
        public string GetAuthorizationUrl(string authUrl, string requestKey, string callbackUrl)
        {
            Checker.Parameter(!string.IsNullOrWhiteSpace(authUrl), "authUrl can not be empty or null");
            Checker.Parameter(!string.IsNullOrWhiteSpace(requestKey), "requestKey can not be empty or null");
            Checker.Parameter(!string.IsNullOrWhiteSpace(callbackUrl), "callbackUrl can not be empty or null");

            return string.Format("{0}?{1}={2}&{3}={4}", authUrl, OAuthTokenKey, requestKey, OAuthCallbackKey, HttpUtility.UrlEncode(callbackUrl));

        }

        /// <summary>
        /// Gets the access credential.
        /// </summary>
        /// <param name="url">The api url.</param>
        /// <param name="verifier">The verifier.</param>
        /// <param name="tempCredential">The temporary credential.</param>
        /// <param name="additionalParams">The additional parameters callback from the api service.</param>
        /// <returns>
        /// The access credential
        /// </returns>
        public Credential GetAccessCredential(string url, string verifier, Credential tempCredential, out NameValueCollection additionalParams)
        {
            Checker.Parameter(!string.IsNullOrWhiteSpace(url), "url can not be empty or null");
            Checker.Parameter(!string.IsNullOrWhiteSpace(verifier), "verifier can not be empty or null");

            string normalizedUrl;
            string normalizedRequestParameters;
            string sign = GenerateSignature(new Uri(url), tempCredential.Key, tempCredential.Secret, verifier, HttpMethod.Get, SignatureTypes.HMACSHA1, out normalizedUrl, out normalizedRequestParameters);

            string requestUrl = string.Format("{0}?{1}&{2}={3}", normalizedUrl, normalizedRequestParameters, OAuthSignatureKey, HttpUtility.UrlEncode(sign));

            ResponseObj respObj = WebHost.HttpGet(requestUrl);

            Checker.Requires(respObj.Code == System.Net.HttpStatusCode.OK, "request error, code: {0} text: {1}", respObj.Code, respObj.Text);

            NameValueCollection paramCollection = HttpKits.ResolveParameters(respObj.Text);

            Checker.Requires(paramCollection.Count >= 2, "response not match the protocol, text: {0}", respObj.Text);

            Credential c= new Credential { Key = paramCollection[OAuthTokenKey], Secret = paramCollection[OAuthTokenSecretKey] };

            paramCollection.Remove(OAuthTokenKey);
            paramCollection.Remove(OAuthTokenSecretKey);

            additionalParams = paramCollection;

            return c;
        }
    }
}
